const std = @import("std");

// Although this function looks imperative, note that its job is to
// declaratively construct a build graph that will be executed by an external
// runner.
pub fn build(b: *std.Build) void {
	// Standard target options allows the person running `zig build` to choose
	// what target to build for. Here we do not override the defaults, which
	// means any target is allowed, and the default is native. Other options
	// for restricting supported target set are available.
	const target = b.standardTargetOptions(.{});

	// Standard optimization options allow the person running `zig build` to select
	// between Debug, ReleaseSafe, ReleaseFast, and ReleaseSmall. Here we do not
	// set a preferred release mode, allowing the user to decide how to optimize.
	const optimize = b.standardOptimizeOption(.{});

	// Create PGMQL module.
	const pgmql = b.addModule("pgmql", .{
		.root_source_file = b.path("lib/root.zig"),
		.target = target,
		.optimize = optimize,
	});

	// Add tardy dependency.
	const tardy = b.dependency("tardy", .{
		.target = target,
		.optimize = optimize,
	});
	pgmql.addImport("tardy", tardy.module("tardy"));

	// Add zzz dependency.
	const zzz = b.dependency("zzz", .{
		.target = target,
		.optimize = optimize,
	});
	pgmql.addImport("zzz", zzz.module("zzz"));

	// Create MQL executable.
	const mql = b.addExecutable(.{
		.name = "mql",
		.root_source_file = b.path("src/mql.zig"),
		.target = target,
		.optimize = optimize,
	});
	mql.root_module.addImport("pgmql", pgmql);
	b.installArtifact(mql);

	// Create example executable.
	const example = b.addExecutable(.{
		.name = "example",
		.root_source_file = b.path("tests/example.zig"),
		.target = target,
		.optimize = optimize,
	});
	example.root_module.addImport("pgmql", pgmql);
	example.root_module.addImport("tardy", tardy.module("tardy"));
	example.root_module.addImport("zzz", zzz.module("zzz"));
	b.installArtifact(example);

	const lib_unit_tests = b.addTest(.{
		.root_source_file = b.path("lib/root.zig"),
		.target = target,
		.optimize = optimize,
	});
	const run_lib_unit_tests = b.addRunArtifact(lib_unit_tests);

	const lib_example_tests = b.addTest(.{
		.root_source_file = b.path("tests/root.zig"),
		.target = target,
		.optimize = optimize,
	});
	// Add pgmql dependency.
	lib_example_tests.root_module.addImport("pgmql", pgmql);
	const run_lib_example_tests = b.addRunArtifact(lib_example_tests);
	run_lib_example_tests.has_side_effects = true;

	const test_step = b.step("test", "Run unit tests.");
	test_step.dependOn(&run_lib_example_tests.step);
	test_step.dependOn(&run_lib_unit_tests.step);
}
